Stan Math Library: stan/math/prim/scal/prob/pareto_cdf.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

pareto_cdf.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_PARETO_CDF_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_PARETO_CDF_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/constants.hpp>
13 #include <stan/math/prim/scal/fun/multiply_log.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <boost/random/exponential_distribution.hpp>
16 #include <boost/random/variate_generator.hpp>
17 #include <cmath>
18 #include <limits>
19
20
21 namespace stan {
22 namespace math {
23
24 template <typename T_y, typename T_scale, typename T_shape>
25 typename return_type<T_y, T_scale, T_shape>::type
26 pareto_cdf(const T_y& y, const T_scale& y_min, const T_shape& alpha) {
27 typedef typename stan::partials_return_type<T_y, T_scale, T_shape>::type
28 T_partials_return;
29
30 // Check sizes
31 // Size checks
32 if ( !( stan::length(y) && stan::length(y_min) && stan::length(alpha) ) )
33 return 1.0;
34
35 // Check errors
36 static const char* function("stan::math::pareto_cdf");
37
38 using stan::math::check_positive_finite;
39 using stan::math::check_not_nan;
40 using stan::math::check_greater_or_equal;
41 using stan::math::check_consistent_sizes;
42 using stan::math::check_nonnegative;
43 using stan::math::value_of;
44 using std::log;
45 using std::exp;
46
47 T_partials_return P(1.0);
48
49 check_not_nan(function, "Random variable", y);
50 check_nonnegative(function, "Random variable", y);
51 check_positive_finite(function, "Scale parameter", y_min);
52 check_positive_finite(function, "Shape parameter", alpha);
53 check_consistent_sizes(function,
54 "Random variable", y,
55 "Scale parameter", y_min,
56 "Shape parameter", alpha);
57
58 // Wrap arguments in vectors
59 VectorView<const T_y> y_vec(y);
60 VectorView<const T_scale> y_min_vec(y_min);
61 VectorView<const T_shape> alpha_vec(alpha);
62 size_t N = max_size(y, y_min, alpha);
63
64 OperandsAndPartials<T_y, T_scale, T_shape>
65 operands_and_partials(y, y_min, alpha);
66
67 // Explicit return for extreme values
68 // The gradients are technically ill-defined, but treated as zero
69
70 for (size_t i = 0; i < stan::length(y); i++) {
71 if (value_of(y_vec[i]) < value_of(y_min_vec[i]))
72 return operands_and_partials.value(0.0);
73 }
74
75 // Compute vectorized CDF and its gradients
76
77 for (size_t n = 0; n < N; n++) {
78 // Explicit results for extreme values
79 // The gradients are technically ill-defined, but treated as zero
80 if (value_of(y_vec[n]) == std::numeric_limits<double>::infinity()) {
81 continue;
82 }
83
84 // Pull out values
85 const T_partials_return log_dbl = log(value_of(y_min_vec[n])
86 / value_of(y_vec[n]));
87 const T_partials_return y_min_inv_dbl = 1.0 / value_of(y_min_vec[n]);
88 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
89
90 // Compute
91 const T_partials_return Pn = 1.0 - exp(alpha_dbl * log_dbl);
92
93 P *= Pn;
94
95 if (!is_constant_struct<T_y>::value)
96 operands_and_partials.d_x1[n]
97 += alpha_dbl * y_min_inv_dbl * exp((alpha_dbl + 1) * log_dbl)
98 / Pn;
99 if (!is_constant_struct<T_scale>::value)
100 operands_and_partials.d_x2[n]
101 += - alpha_dbl * y_min_inv_dbl * exp(alpha_dbl * log_dbl) / Pn;
102 if (!is_constant_struct<T_shape>::value)
103 operands_and_partials.d_x3[n]
104 += - exp(alpha_dbl * log_dbl) * log_dbl / Pn;
105 }
106
107 if (!is_constant_struct<T_y>::value) {
108 for (size_t n = 0; n < stan::length(y); ++n)
109 operands_and_partials.d_x1[n] *= P;
110 }
111 if (!is_constant_struct<T_scale>::value) {
112 for (size_t n = 0; n < stan::length(y_min); ++n)
113 operands_and_partials.d_x2[n] *= P;
114 }
115 if (!is_constant_struct<T_shape>::value) {
116 for (size_t n = 0; n < stan::length(alpha); ++n)
117 operands_and_partials.d_x3[n] *= P;
118 }
119
120 return operands_and_partials.value(P);
121 }
122 }
123 }
124 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_greater_or_equal
bool check_greater_or_equal(const char *function, const char *name, const T_y &y, const T_low &low)
Return true if y is greater or equal than low.
Definition: check_greater_or_equal.hpp:84
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
partials_return_type.hpp
OperandsAndPartials.hpp
constants.hpp
stan::math::pareto_cdf
return_type< T_y, T_scale, T_shape >::type pareto_cdf(const T_y &y, const T_scale &y_min, const T_shape &alpha)
Definition: pareto_cdf.hpp:26
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
check_greater_or_equal.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.